from django.conf.urls import url
from . import views

app_name = 'telegram_bots'
urlpatterns = [
    url(
        regex=r'^$',
        view=views.BotListView.as_view(),
        name='list',
    ),
    url(
        regex=r'^(?P<pk>\d+)/$',
        view=views.BotDetailView.as_view(),
        name='detail',
    ),
    url(
        regex=r'^~create/$',
        view=views.BotCreateView.as_view(),
        name='create',
    ),
    url(
        regex=r'^~update/$',
        view=views.BotUpdateView.as_view(),
        name='update',
    ),
    url(
        regex=r'^~delete/$',
        view=views.BotDeleteView.as_view(),
        name='delete',
    ),
    url(
        regex=r'^~subscribe/(?P<signature>.+)/$',
        view=views.BotSubscribeView.as_view(), 
        name='subscribe'
    ),
    url(
        regex=r'^(?P<bot_token>.+)/$', 
        view=views.CommandReceiveView.as_view(), 
        name='auth'
    ),
]


from django.contrib import admin
from .models import Bot, TelegramUser
# Register your models here.

@admin.register(Bot)
class BotAdmin(admin.ModelAdmin):
	fields = ['api_key']

admin.site.register(TelegramUser)


message_schema = {
	'bot': {
		'type': 'int',
		'coerce': int,
		'required': True,
		'empty': False
	},
	'type': {
		'type': 'list',
		'allowed': ['HTML', 'Markdown', 'Text'],
		'required': True,
		'empty': False
	},
	'message': {
		'type': 'string',
		'minlength': 1,
		'required': True,
		'empty': False
	},
	'chat_ids': {
		'type': 'list',
		'required': True,
		'empty': False
	}
}



# Generated by Django 2.0.2 on 2018-03-23 16:58

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('telegram_bots', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='authorization',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='authorizations', to='telegram_bots.TelegramUser'),
        ),
    ]



from django.contrib.auth.mixins import LoginRequiredMixin

from django.shortcuts import get_object_or_404, redirect

from django.http import HttpResponseBadRequest, JsonResponse
from django.http.response import HttpResponse

from django.views.generic import View, ListView, CreateView, UpdateView, DeleteView, DetailView, RedirectView, FormView
from django.views.decorators.csrf import csrf_exempt

from django.utils.decorators import method_decorator
from django.utils.translation import ugettext_lazy as _


from .utils import extract_command, decode_signin, get_bot_model, get_telegram_user_model
from .signals import authorize_user, receive_message, receive_callback_query, receive_command
from .forms import MessageForm

import ujson as json


Bot = get_bot_model()
TelegramUser = get_telegram_user_model()


class BotListView(LoginRequiredMixin, ListView):

    model = Bot
    def get_queryset(self):
        queryset = super(BotListView, self).get_queryset()
        return  queryset.filter(signal_sender__user=self.request.user)


class BotDetailView(LoginRequiredMixin, DetailView):

    model = Bot


class BotCreateView(LoginRequiredMixin, CreateView):
    
    model = Bot
    fields = ['api_key']


class BotUpdateView(LoginRequiredMixin, UpdateView):
    
    model = Bot
    fields = ['api_key']


class BotDeleteView(LoginRequiredMixin, DeleteView):

    model = Bot


class CommandReceiveView(View):

    def post(self, request, bot_token):
        bot = get_object_or_404(Bot, api_key=bot_token)
        body_unicode = request.body.decode('utf-8')
        payload = json.loads(json.dumps(body_unicode))
        try:
            payload = json.loads(request.body.decode('utf-8'))
        except ValueError:
            return HttpResponseBadRequest('Invalid request body')
        else:
            message = payload.get('message', None)
            callback_query = payload.get('callback_query', None)
            if message:
                chat_id = payload['message']['chat']['id']
                #type = payload['message']['chat']['type']
                text = payload['message']['text'] # command
                message = payload['message']
                receive_message.send(sender=Bot, bot_id=bot.bot_id, user_id=chat_id, text=text, message=message)
                command = extract_command(text)
                if command:
                    receive_command.send(sender=Bot, bot_id=bot.bot_id, user_id=chat_id, command=command, payload=text)
            if callback_query:
                receive_callback_query.send(sender=Bot, bot_id=bot.bot_id, user_id=callback_query['from']['id'], data=callback_query['data'])
        return JsonResponse({}, status=200)

    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        return super(CommandReceiveView, self).dispatch(request, *args, **kwargs)


class BotSubscribeView(LoginRequiredMixin, RedirectView):

    def get(self, request, signature):
        data = decode_signin(signature)
        if data:
            user = get_object_or_404(TelegramUser, user__username=data['user_username'])
            bot = get_object_or_404(Bot, username=data['bot_username'])
        try:
            authorize_user.send(
                sender = Bot, 
                key = data['key']
            )
            return redirect('https://telegram.me/{}?start={}'.format(bot.username, user.unique_code))
        except Exception as e:
            return HttpResponse(e)

        return super(BotSubscribeView, self).get(request, signature)


class BotUnsubscribeView(LoginRequiredMixin, RedirectView):

    def get(self, request):

        user = self.request.user
        telegramuser = Telegramuser.objects.filter(user=user)
        if telegramuser:
            telegramuser.delete()

        return super(BotSubscribeView, self).get(request)


class BotRefresUserUniqueCode(LoginRequiredMixin, RedirectView):

    def get(self, request):

        user = self.request.user
        telegramuser = Telegramuser.objects.filter(user=user)
        if telegramuser:
            telegramuser.set_unique_code
            telegramuser.authorization.is_activate = False
            telegramuser.authorization.save()
            telegramuser.save()

        return super(BotRefresUserUniqueCode, self).get(request)


class SendMessageView(LoginRequiredMixin, FormView):
    form_class = MessageForm

    def post(self, request, *args, **kwargs):
        
        bot = request.POST.get('bot', None)
        type = request.POST.get('type', None)
        message = request.POST.get('message', None)
        
        if bot and type and message:
            print(bot)
            print(type)
            print(message)
            bot.sendMessage()
        return super(SendMessageView, self).post(request, *args, **kwargs)



from rest_framework import permissions

class IsOwnerOrReadOnly(permissions.BasePermission):
    """
    Object-level permission to only allow owners of an object to edit it.
    Assumes the model instance has an `user` attribute.
    """

    def has_object_permission(self, request, view, obj):
        # Read permissions are allowed to any request,
        # so we'll always allow GET, HEAD or OPTIONS requests.
        if request.method in permissions.SAFE_METHODS:
            return True

        # Instance must have an attribute named `owner` or 'user'.
        if getattr(obj, 'owner', False):
            return obj.owner == request.user
        else:
            return obj.user == request.user


from rest_framework import viewsets
from rest_framework import status
from rest_framework.response import Response
from rest_framework import mixins
from rest_framework.decorators import detail_route, list_route
from rest_framework.permissions import IsAuthenticatedOrReadOnly

from .models import Bot, TelegramUser, Authorization
from .serializers import BotListSerializer, BotSerializer, MessageSerializer, TelegramUserSerializer, AuthorizationSerializer
from .permissions import IsOwnerOrReadOnly
import ujson as json

from .utils import extract_command, decode_signin, get_bot_model, get_telegram_user_model
from .signals import authorize_user, receive_message, receive_callback_query, receive_command
from .validators import message_schema

from django.shortcuts import get_object_or_404

from cerberus import Validator

v = Validator()


class WebhookViewSet(viewsets.ViewSet):

	@detail_route(methods=['post'])
	def webhook(self, request, api_key):

		bot = get_object_or_404(Bot, api_key=api_key)
		body_unicode = request.body.decode('utf-8')
		payload = json.loads(json.dumps(body_unicode))
		try:
			payload = json.loads(request.body.decode('utf-8'))
		except ValueError:
			return Response('Invalid request body', status=status.HTTP_400_BAD_REQUEST) #HttpResponseBadRequest('Invalid request body')
		else:
			message = payload.get('message', None)
			callback_query = payload.get('callback_query', None)
			if message:
				chat_id = payload['message']['chat']['id']
				#type = payload['message']['chat']['type']
				text = payload['message']['text'] # command
				message = payload['message']
				receive_message.send(sender=Bot, bot_id=bot.bot_id, user_id=chat_id, text=text, message=message)
				command = extract_command(text)
				if command:
					receive_command.send(sender=Bot, bot_id=bot.bot_id, user_id=chat_id, command=command, payload=text)
			if callback_query:
				receive_callback_query.send(sender=Bot, bot_id=bot.bot_id, user_id=callback_query['from']['id'], data=callback_query['data'])
		return Response(status=status.HTTP_200_OK)


class SubscribeViewSet(viewsets.ViewSet):

	@detail_route(methods=['post'])
	def subscribe(self, request, signature):
		data = decode_signin(signature)
		if data:
			user = get_object_or_404(TelegramUser, user__username=data['user_username'])
			bot = get_object_or_404(Bot, username=data['bot_username'])
		try:
			authorize_user.send(
				sender = Bot, 
				key = data['key']
			)
			response = {'link': 'https://telegram.me/{}?start={}'.format(bot.username, user.token)}
			return Response(response, status=status.HTTP_200_OK)
		except Exception as e:
			return Response(e, status=status.HTTP_400_BAD_REQUEST)
		content = {"signature": ["This field is required."]}
		return Response(content, status=status.HTTP_400_BAD_REQUEST)


class BotViewSet(viewsets.ModelViewSet):

	queryset = Bot.objects.all()

	def get_serializer_class(self):
		if not self.action:
			print('hello')
		if self.action == 'list':
			return BotListSerializer
		else:
			return BotSerializer

	def get_permission_classes(self):
		print(self.action)
		if self.action == 'list':
			permission_classes = [IsAuthenticatedOrReadOnly]
		else:
			permission_classes = [IsOwnerOrReadOnly]
		return [permission() for permission in permission_classes]

	@detail_route(methods=['post'])
	def send_message(self, request, pk=None):
		serializer = MessageSerializer(data=request.data)		
		if serializer.is_valid():
			bot = self.get_object()
			users_list = [user_id for user_id in serializer.data['chat_ids'] if user_id in bot.active_auth_user_ids]
			for user_id in users_list:
				bot.send_message(chat_id, serializer.data['payload'])
			return Response(serializer.data, status=status.HTTP_201_CREATED)
		return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

	@detail_route(methods=['post'])
	def unsubscribe(self, request, pk=None):
		bot = get_object_or_404(Bot, pk=pk)
		user = self.request.user.telegramuser
		#user = get_object_or_404(TelegramUser, user=self.request.user)
		authorization = Authorization.objects.filter(user=user, bot=bot).first()
		if authorization:
			if authorization.is_active:
				authorization.is_active = False
				authorization.save()
				content = {'message': 'You successfully unsubscribed from this bot'}
				return Response(content, status=status.HTTP_200_OK)
		content = {'message': 'You are not a subscriber on this bot'}
		return Response(content, status=status.HTTP_404_NOT_FOUND)

	def create(self, request):
		if 'api_key' in request.data:
			try:
				bot, created = Bot.objects.get_or_create(api_key=request.data['api_key'], owner=self.request.user)
				serializer = self.get_serializer(bot)
				return Response(serializer.data, status=status.HTTP_201_CREATED)
			except Exception as e:
				content = {"message": "Bot with this api key not found"}
				error = json.dumps(e.json)
				message = json.loads(error)
				message.update(content)
				return Response(message, status=status.HTTP_404_NOT_FOUND)
		content = {"api_key": ["This field is required."]}
		return Response(content, status=status.HTTP_400_BAD_REQUEST)


class TelegramUserViewSet(viewsets.ModelViewSet):

	'''
	permissions = IsAuthenticatedOrReadOnly
	queryset = TelegramUser.objects.all()
	serializer_class = TelegramUserSerializer
	'''
	queryset = TelegramUser.objects.all()
	serializer_class = TelegramUserSerializer
	permission_classes = [IsOwnerOrReadOnly]

	@detail_route(methods=['post'])
	def reset_token(self, request, pk=None):
		user = self.get_object()
		if user:
			user.set_token
			serializer = TelegramUserSerializer(user)
			for user in user.authorizations.filter(is_active=True):
				user.is_active = False
				user.save()
			response = {'status': 'The token is reset, all accounts are deactivated'}
			return Response(response, status=status.HTTP_200_OK)
		return Response(status=status.HTTP_404_NOT_FOUND)


class AuthorizationViewSet(viewsets.ModelViewSet):
	'''
	permissions = IsAuthenticatedOrReadOnly
	queryset = Authorization.objects.all()
	serializer_class = AuthorizationSerializer
	'''
	queryset = Authorization.objects.all()
	serializer_class = AuthorizationSerializer
	permission_classes = [IsOwnerOrReadOnly]

	@detail_route(methods=['post'])
	def reset_token(self, request, pk=None):
		'''
		user = self.request.user
        telegramuser = Telegramuser.objects.filter(user=user)
        if telegramuser:
            telegramuser.set_unique_code
            telegramuser.authorization.is_activate = False
            telegramuser.authorization.save()
            telegramuser.save()
		'''
		return Response(status=status.HTTP_200_OK)


from rest_framework import serializers
from .models import Bot, TelegramUser, Authorization
from rest_framework.utils import html, humanize_datetime, json, representation
import collections


class BotListSerializer(serializers.ModelSerializer):

	class Meta:
		model = Bot
		fields = ['first_name', 'username', 'avatar']
		retrieve_fields = ['id']


class MessageSerializer(serializers.Serializer):
	
	type = serializers.ChoiceField(required=True, choices=['HTML', 'Markdown', 'Text'])
	payload = serializers.CharField(required=True, max_length=2000)
	chat_ids = serializers.ListField(min_length=1, max_length=10000, required=True,
		child=serializers.IntegerField(min_value=1, max_value=10000000000))


class BotSerializer(serializers.ModelSerializer):

	class Meta:
		model = Bot
		fields = '__all__'
		retrieve_fields = ['id']


class TelegramUserSerializer(serializers.ModelSerializer):

	class Meta:
		model = TelegramUser
		exclude = ['token', 'chat_id']


class AuthorizationSerializer(serializers.ModelSerializer):

	class Meta:
		model = Authorization
		fields = '__all__'


from django.dispatch import receiver
from django.db.models.signals import post_save
from django.contrib.auth import get_user_model
from django.dispatch import Signal
from .utils import extract_data_from_command, get_bot_model, get_telegram_user_model, get_authorization_model
from .services import get_user_from_storage, activate_user


User = get_user_model()
Bot = get_bot_model()
TelegramUser = get_telegram_user_model()
Authentication = get_authorization_model()


receive_message = Signal(providing_args=['bot_id', 'chat_id', 'type', 'text', 'message'])
receive_command = Signal(providing_args=['bot_id', 'chat_id', 'type', 'command', 'payload', 'chat_id'])
receive_callback_query = Signal(providing_args=['bot_id', 'chat_id', 'data'])
authorize_user = Signal(providing_args=['key'])


@receiver(post_save, sender=User)
def create_bot_user(sender, instance, created, **kwargs):
	if created:
		TelegramUser.objects.create(user=instance)


@receiver(receive_command, sender=Bot)
def authentication_user(sender, **kwargs):
	if all(k in kwargs for k in ('command', 'payload', 'chat_id', 'type')):
		command = kwargs['command']
		payload = kwargs['payload']
		chat_id = kwargs['chat_id']
		type = kwargs['type']

		if command == '/start' and payload:
			unique_code = extract_data_from_command(payload)
			if unique_code:
				user = get_user_from_storage(unique_code)
				if user:
					if not user.is_authorized:
						activate_user(user.id, bot_id)
						save_chat_id(chat_id, user)
						reply = 'Hi, {}, the authentication was successful!'.format(user.user.username)
					else:
						reply = '{}, you are already authenticated'.format(user.user.username)
				else:
					reply = 'Unfortunately, I can not authenticate you :('




# Generated by Django 2.0.2 on 2018-03-22 15:48

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Authorization',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_active', models.BooleanField(default=False, verbose_name='Is active')),
            ],
        ),
        migrations.CreateModel(
            name='Bot',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('api_key', models.CharField(max_length=255, unique=True, verbose_name='API_KEY')),
                ('chat_id', models.CharField(max_length=255, verbose_name='Id')),
                ('first_name', models.CharField(max_length=255, verbose_name='First name')),
                ('username', models.CharField(max_length=255, verbose_name='Username')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='bots', to=settings.AUTH_USER_MODEL, verbose_name='Owner')),
            ],
            options={
                'verbose_name': 'Bot',
                'verbose_name_plural': 'Bots',
            },
        ),
        migrations.CreateModel(
            name='TelegramUser',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('token', models.CharField(max_length=255, verbose_name='Token')),
                ('chat_id', models.CharField(max_length=255, null=True, verbose_name='User id')),
                ('user', models.OneToOneField(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='telegramuser', to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
        ),
        migrations.AddField(
            model_name='authorization',
            name='bot',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='authorizations', to='telegram_bots.Bot'),
        ),
        migrations.AddField(
            model_name='authorization',
            name='user',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='authorization', to='telegram_bots.TelegramUser'),
        ),
        migrations.AlterUniqueTogether(
            name='authorization',
            unique_together={('bot', 'user')},
        ),
    ]



from django import forms
from .utils import get_bot_model, get_telegram_user_model
from django.utils.translation import ugettext_lazy as _


Bot = get_bot_model()
User = get_telegram_user_model()

class MessageForm(forms.Form):
	bot = forms.ModelChoiceField(queryset=[], label=_(''))
	type = forms.ChoiceField(choices=['HTML', 'TEXT'])
	users = forms.ModelChoiceField(queryset=[], label=_(''))
	message = forms.CharField(widget=forms.Textarea)

	def __init__(self, request, *args, **kwargs):
		super(MessageForm, self).__init__(*args, **kwargs)
		if request.user:
			bot_queryset = Bot.objects.filter(owner=request.user)
			self.fields['bot'].queryset = bot_queryset
			self.fields['bot'].label = _('Select Telegram bot')

			if bot_queryset:
				users_queryset = []
				for bot in bot_queryset:
					for authorization in bot.authorizations.all():
						users_queryset.append(authorization.user)
				self.fields['users'].queryset = users_queryset
				self.fields['users'].label = _('Select users to send the message')


import telepot
import environ
import string
import random
from django.db import models
from django.urls import reverse
from django.conf import settings
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from telepot.loop import OrderedWebhook
from collections import defaultdict
from django.utils.functional import cached_property


env = environ.Env()

@python_2_unicode_compatible
class Bot(models.Model):
	
	api_key = models.CharField(max_length=255, verbose_name=_('API_KEY'), unique=True)
	chat_id = models.CharField(max_length=255, verbose_name=_('Id'))
	first_name = models.CharField(max_length=255, verbose_name=_('First name'))
	username = models.CharField(max_length=255, verbose_name=_('Username'))

	owner = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_('Owner'), related_name='bots', on_delete=models.CASCADE)

	@property
	def get_me(self):
		data = self.bot.getMe()
		self.chat_id = data['id']
		self.first_name = data['first_name']
		self.username = data['username']
		return data

	@property
	def bot(self):
		return telepot.Bot(self.api_key)

	@property
	def active_auth_user_ids(self):
		return [user.id for user in self.authorizations.filter(is_active=True)]

	def send_message(self, chat_id, payload, parse_mode=None, reply_markup=None):
		bot = self.bot
		message = bot.sendMessage(chat_id, payload, parse_mode=parse_mode, reply_markup=reply_markup)
		return message

	def save(self, *args, **kwargs):
		if not self.pk:
			self.get_me
			#self.bot.setWebhook('https://{domain}/bots/{bot_token}/'.format(domain=env('DOMAIN_NAME', default='stagingserver.xyz'), bot_token=self.api_key))
		return super(Bot, self).save(*args, **kwargs)

	def delete(self, *args, **kwargs):
		self.bot.deleteWebhook()
		return super(Bot, self).delete(*args, **kwargs)

	@property
	def photos(self):
		photos = defaultdict()
		all_photos = self.bot.getUserProfilePhotos(self.chat_id)
		for photo_counter in range(all_photos['total_count']):
			for photo in all_photos['photos'][photo_counter]:
				if not photo in photos:
					photos[photo] = defaultdict()
				file = self.bot.getFile(photo['file_id'])
				photos[photo] = defaultdict({
					photo['height']: 'https://api.telegram.org/file/bot{}/{}'.format(
						self.api_key,
						file['file_path']
						)
					})
		return photos

	@cached_property
	def avatar(self):
		if self.photos:
			for photos in self.photos:
				for photo in self.photos[photos]:
					return self.photos[photos][photo]
		return None

	class Meta:
		verbose_name = _('Bot')
		verbose_name_plural = _('Bots')

	def __str__(self):
		return '%s' % str(self.first_name)


@python_2_unicode_compatible
class TelegramUser(models.Model):

	user = models.OneToOneField(settings.AUTH_USER_MODEL, null=True, verbose_name=_('User'), related_name='telegramuser', on_delete=models.CASCADE)
	token = models.CharField(max_length=255, verbose_name=_('Token'))
	chat_id = models.CharField(max_length=255, verbose_name=_('Chat id'), null=True, blank=False)

	@property
	def generate_token(self, size=64, chars=string.ascii_uppercase + string.digits + string.ascii_lowercase):
		token = ''.join(random.choice(chars) for _ in range(size))
		return token

	@property
	def set_token(self):
		self.token = self.generate_token

	def check_token(self, token):
		return self.token == token

	def save(self, *args, **kwargs):
		if not self.pk or not self.token:
			self.set_token
		return super(TelegramUser, self).save(*args, **kwargs)

	def __str__(self):
		return '%s' % str(self.user.username)


@python_2_unicode_compatible
class Authorization(models.Model):

	bot = models.ForeignKey(Bot, on_delete=models.CASCADE, verbose_name=_('Bot'), related_name='authorizations')
	user = models.ForeignKey(TelegramUser, on_delete=models.CASCADE, verbose_name=_('User'), related_name='authorizations')
	is_active = models.BooleanField(default=False, verbose_name=_('Is active'))

	@property
	def activation_url(self):
		from .utils import siging_auth
		mamespase = env('TELEGRAM_BOTS_NAMESPACE', default='telegram_bots')
		signature = siging_auth(self.bot.username, self.user.user.username, self.pk)
		return reverse('{}:subscribe/{}'.format(mamespase, signature))


	class Meta:
		unique_together = (('bot', 'user'),)


from django.core import signing
from django.core.signing import BadSignature, SignatureExpired
import logging


logger = logging.getLogger(__name__)


def extract_data_from_command(text):
	return text.split()[1] if len(text.split()) > 1 else None

def extract_command(text):
	return text.split()[0] if text.split()[0].startswith('/', 0, 1) is True else None

def siging_auth(bot_username, user_username, key):
	signature = signing.dumps({
		'bot_username': bot_username,
		'user_username': user_username,
		'key': key
		})
	return signature

def decode_signin(signature):
	sign = None
	try:
		sign = signing.loads(signature, max_age=60*60*24*30)
	except SignatureExpired:
		logger.exception("signature expired")
	except BadSignature:
		logger.exception("signature invalid")
	return sign


def get_bot_model():
	from .models import Bot
	return Bot

def get_telegram_user_model():
	from .models import TelegramUser
	return TelegramUser

def get_authorization_model():
	from .models import Authorization
	return Authorization


from rest_framework import routers
from .viewsets import BotViewSet, TelegramUserViewSet, AuthorizationViewSet, WebhookViewSet
from django.urls import re_path

app_name = 'api_telegram_bots'

router = routers.SimpleRouter()
router.register(r'^(?P<api_key>.+)/$', WebhookViewSet, base_name='webhook')
router.register(r'bots', BotViewSet)
router.register(r'users', TelegramUserViewSet)
router.register(r'authorizations', AuthorizationViewSet)
urlpatterns = router.urls


from .models import TelegramUser
from annoying.functions import get_object_or_None
from .utils import get_authorization_model, get_bot_model, get_telegram_user_model


Bot = get_bot_model()
User = get_telegram_user_model()
Authorization = get_authorization_model()


def get_user_from_storage(unique_code): 
	user = get_object_or_None(TelegramUser, unique_code=unique_code)
	return user	

def activate_user(user_id, bot_id):
	bot = get_object_or_None(Bot, bot_id=bot_id)
	user = get_object_or_None(User, id=user_id)

	if user and bot:
		authorization, created = Authorization.objects.get_or_create(bot=bot, user=user)
		if not authorization.is_active:
			authorization.is_active = True
			authorization.save()

def deactivate_user(user_id, bot_id):
	bot = get_object_or_None(Bot, bot_id=bot_id)
	user = get_object_or_None(User, id=user_id)

	if user and bot:
		authorization, created = Authorization.objects.get_or_create(bot=bot, user=user)
		if not authorization.is_active:
			authorization.is_active = False
			authorization.save()

def save_chat_id(chat_id, user):
	telegram_user, created = User.objects.get_or_create(user)
	#chat, created = Chat.objects.get_or_create(chat_id=chat_id)
	#chat.type = type
	telegram_user.chat_id = chat_id
	chat.save()
	return chat